% Note:
% The code that you are using is free.
% 
% This code is for analysis of lamellar gratings in TM polarization. In 
% this code adaptive spatial resolution technique is employed to provide an
% excellent convergence rate. The code has been written based on [1]. The 
% jump points are also regularized (see [2]).
% 
% [1] A. Khavasi and K. Mehrany, Adaptive Spatial Resolution in Fast,
% Efficient, and Stable Analysis of Metallic Lamellar Gratings at Microwave
% Frequencies, IEEE Transaction on Antenna and Propag., 57, 1115-1127 (2009).
% 
% [2] A. Khavasi, K. Mehrany, Regularization of jump points in applying 
% the adaptive spatial resolution technique, Opt. Commun., 284, 3211-3215 (2011), 
% doi:10.1016/j.optcom.2011.03.037 
% 
% Inputs:
% 
% N: Truncation order
% lambda: Free space wavelength normalized to grating period
% theta: incident angle
% d: Grating thickness normalized to grating period
% n1: Refractive index of incident medium
% n3: Refractive index of transmission medium
% xl: Transition (discontinuity) points in profile of refractive index 
% nl: The refractive index between transitions
% 
% The outputs of this program are diffraction efficiencies which are stored
% in vectors 'DE1' and 'DE3' for reflected and transmitted waves,
% respectively. For example for N = 2 we have:
% DE1 = diffraction efficiency of [2 1 0 -1 -2]'th reflected order
% DE3 = diffraction efficiency of [2 1 0 -1 -2]'th transmitted order
% 
% Example:
% The following figure is one period of a multilayer lamellar grating. This
% grating is defined as follows:
% xl = [0 0.2 0.5 0.8 1]; 
% nl = [n22 n21 n22 n21
%       n21 n21 n22 n22
%       n23 n23 n23 n23];
% dlayer = [d1 d2 d3];
% 
%   Incident plane wave
%                   \    |
%                    \   |
%                     \  |   Region 1 (n1)
%                      \ |
%       -    -----------------------------------
%      |     |       |         |        |      |
%  d1 <      |  n22  |   n21   |   n22  |  n21 |
%      |     |       |         |        |      |
%       -    -----------------------------------
%      |     |                 |               |
%  d2 <      |      n21        |       n22     |
%      |     |                 |               |
%       -    -----------------------------------
%      |     |                                 |
%  d3 <      |                n23              |
%      |     |                                 |
%       -    -----------------------------------
%          x1=0    x2=0.2    x3=0.5   x4=0.8 x5=1
%            <--------------------------------->
%                    Grating Period(=1)         
%                  
%                         Region 3 (n3)
% 
% The default parameteres in this code have been set according to the first
% resonance in Fig. 6(c) of the following article:
% Amin Khavasi, Masoud Edalatipour, Khashayar Mehrany: "Circuit Model for
% Extraordinary Transmission Through Periodic Array of Subwavelength Stepped
% Slits." IEEE Transactions on Antennas and Propagation 04/2013; 61(4)
% :2019-2024. DOI:10.1109/TAP.2012.2237152

% % % % % 
%Inputs
% % % % % 
N = 30;   % Truncation order: Conevergence parameter. N = 30 is large enough for almost all structures
lambda = 1.4/0.0785; % Free space wavelength normalized to grating period
n1 = 1.4;   % Refractive index of incident medium
n3 = 1.4; % Refractive index of transmission medium
theta = 0; % incident angle
% -----------------------------------
% Defining grating region parameteres
% -----------------------------------
% Transition (discontinuity) points in profile of refractive index are
% defined by 'xl'. You must set first and last element of 'xl' to 0 and 1,
% repectively. It means that the first transition occurs at the begining of
% the grating period. 
 xl = [0 0.25 0.35 0.6 1];

% The refractive index between transitions is determined by 'nl'. 'nl(m)' 
% is the refractive index of the region between xl(m) and xl(m+1).
sigma=3.7e5;
n21 = 1;
n22 = sqrt(1-60*sigma*lambda*1i);
   
nl = [n22 n21 n22 n22 
      n21 n21 n21 n22 
      n22 n21 n22 n22];
% Thickness of layers normalized to grating period
dlayer = [2 1 2];

% -------------------
% Initial computaions
% -------------------
sizenl = size(nl);
Nlayer = sizenl(1);
k0=2*pi/lambda;
k1=k0*n1;
k3=k0*n3;
kx=zeros(2*N+1,1);
kx(1:2*N+1)=2*pi*(n1*sin(theta)/lambda-(-N:N));
kz1(1:2*N+1)=sqrt(k1^2-kx(1:2*N+1).^2);
kz3(1:2*N+1)=sqrt(k3^2-kx(1:2*N+1).^2);
for j=1:2*N+1
    if imag(kz1(j))>1e-12
        kz1(j)=-kz1(j);
    end
    if imag(kz3(j))>1e-12
        kz3(j)=-kz3(j);
    end
end
Kx=diag(kx);
Kz1=diag(kz1);
Kz3=diag(kz3);
I=eye(2*N+1);
Zero=zeros(2*N+1);
% --------------------

% --------------
% New coordinate
% --------------
G=0.001;

% -------------------------------------------------------------------------
% In this part, transition points in new coordinate, 'ul's, are chosen in
% an efficient way based on the arqument in Ref. 2
% -------------------------------------------------------------------------
dx = zeros(1,length(xl)-1);
du=dx;
for j=1:(length(xl)-1)
    dx(j)=(xl(j+1)-xl(j))^(1/3);
end
ul=0*xl;
for j=1:(length(xl)-1)
    du(j)=dx(j)/sum(dx);
    ul(j+1)=ul(j)+du(j);
end

if G==1
    ul=xl;
end
epsl=nl.^2;

ulen=10*N;
u=(0:(ulen))/(ulen+1);
x=zeros(1,ulen+1);
h=x+1e-16;
a=x;
b=x;
warning ('off')
for j=2:length(xl)
    a1=(ul(j)*xl(j-1)-ul(j-1)*xl(j))/(ul(j)-ul(j-1));
    a2=(xl(j)-xl(j-1))/(ul(j)-ul(j-1));
    a3=G*(ul(j)-ul(j-1))-(xl(j)-xl(j-1));

    x((ul(j-1)*ulen+1):(ul(j)*ulen+1))=a1+a2*u((ul(j-1)*ulen+1):(ul(j)*ulen+1))+a3*sin(2*pi*(u((ul(j-1)*ulen+1):(ul(j)*ulen+1))-ul(j-1))/(ul(j)-ul(j-1)))/2/pi;
    h((ul(j-1)*ulen+1):(ul(j)*ulen+1))=a2+a3*cos(2*pi*(u((ul(j-1)*ulen+1):(ul(j)*ulen+1))-ul(j-1))/(ul(j)-ul(j-1)))/(ul(j)-ul(j-1));
end
warning ('on')
% End of defining new coordinate
% -------------------------------------------------------------------------

% ------------------------------------------
% Construction of required toeplitz matrices
% ------------------------------------------
hmn=ftaubu(epsl./epsl,ul,xl,G,2*N).';
Hmn=toeplitz(hmn(2*N+1:4*N+1),hmn(2*N+1:-1:1));

invH = inv(Hmn);

% Solving eigenvalue problem in incident region
L1=(k0^2*I)*n1^2-invH*Kx*invH*Kx;
[H1,eigL1]=eig(L1);

% Solving eigenvalue problem in transmission region
H3 = H1;
eigL3 = eigL1 - k0^2*(n1^2-n3^2)*I;
eigLin = eigL1;
Hin = H1;

K=zeros(2*N+1);
for p=1:2*N+1
    kp=(fastft(h.*exp(1i*(kx(p)*x-kx(N+1)*u)),N)).';
    kp(1:2*N+1)=kp((2*N+1):-1:1);
    K(p,:)=kp;
end
if G==1
    K=I;
end

eigL1=eigL1.^.5;
eigL3=eigL3.^.5;
Y1=zeros(1,2*N+1);
for j=1:2*N+1
    if (imag(eigL1(j,j))>1e-10)
        eigL1(j,j)=-eigL1(j,j);
    end
    if (imag(eigL3(j,j))>1e-10)
        eigL3(j,j)=-eigL3(j,j);
    end
    Y1(j)=((eigL1(j,j)));
end
Q1=-1i*(H1/n1^2)*eigL1;
Q3=-1i*(H3/n3^2)*eigL3;

W111=H1;
W112=H1;
W121=Q1;
W122=-Q1;

W311=H3;
W312=H3;
W321=Q3;
W322=-Q3;

for jl = 1 : Nlayer
    d = dlayer(jl);
    epsl = nl(jl,:).^2;
    amn=ftaubu(epsl,ul,xl,G,2*N).';
    bmn=ftaubu(1./epsl,ul,xl,G,2*N).';

    Amn=toeplitz(amn(2*N+1:4*N+1),amn(2*N+1:-1:1));
    Bmn=toeplitz(bmn(2*N+1:4*N+1),bmn(2*N+1:-1:1));

    lul = length(ul);
    Iepsu=fastftptrain(ul(2:(lul-1)),1./epsl,2*N);
    IEE=toeplitz(Iepsu(2*N+1:4*N+1),Iepsu(2*N+1:-1:1));
    invIEE = inv(IEE);

    epsu=fastftptrain(ul(2:(lul-1)),epsl,2*N);
    EE=toeplitz(epsu(2*N+1:4*N+1),epsu(2*N+1:-1:1));
    invEE = inv(EE);
% ------------------------------------------

    % Solving eigenvalue problem in grating region
        Lm=inv(Bmn)*(k0^2*Hmn-Kx*inv(Amn)*Kx);
        [Hj,eigLm]=eig(Lm);
    
    
    eigLm=eigLm.^.5;
    Yj=zeros(1,2*N+1);
    for j=1:2*N+1
        if (imag(eigLm(j,j))>1e-10)
            eigLm(j,j)=-eigLm(j,j);
        end
        Yj(j)=eigLm(j,j);
    end
    
    Qj=-1i*(IEE*Hj)*eigLm;

   
    Wj11=Hj;
    Wj12=Hj;
    Wj21=Qj;
    Wj22=-Qj;


    if jl==1
        s1=inv([Wj11 -W112; Wj21 -W122])*[W111 -Wj12;W121 -Wj22 ];

        Tuu=s1(1:(2*N+1),1:(2*N+1));
        Rud=s1(1:(2*N+1),(2*N+2):(4*N+2));
        Rdu=s1((2*N+2):(4*N+2),1:(2*N+1));
        Tdd=s1((2*N+2):(4*N+2),(2*N+2):(4*N+2));
    else
        sj=[I Zero;Zero expeigLm]*inv([Wj11 -W112; Wj21 -W122])*[W111 -Wj12;W121 -Wj22 ]*[expeigLm Zero;Zero I];
        tuu=sj(1:(2*N+1),1:(2*N+1));
        rud=sj(1:(2*N+1),(2*N+2):(4*N+2));
        rdu=sj((2*N+2):(4*N+2),1:(2*N+1));
        tdd=sj((2*N+2):(4*N+2),(2*N+2):(4*N+2));
        
        Txuu=tuu*inv(I-Rud*rdu)*Tuu;
        Rxud=rud+tuu*Rud*inv(I-rdu*Rud)*tdd;    
        Rxdu=Rdu+Tdd*rdu*inv(I-Rud*rdu)*Tuu;
        Txdd=Tdd*inv(I-rdu*Rud)*tdd;
        Tuu=Txuu;Rud=Rxud;Tdd=Txdd;Rdu=Rxdu;
    end
    W111=Wj11;
    W112=Wj12;
    W121=Wj21;
    W122=Wj22;
    expeigLm=zeros(2*N+1);
    for j=1:2*N+1
        expeigLm(j,j)=exp(-1i*eigLm(j,j)*d);
    end
end
sj=[I Zero;Zero expeigLm]*inv([W311 -Wj12; W321 -Wj22])*[Wj11 -W312;Wj21 -W322 ]*[expeigLm Zero;Zero I];

tuu=sj(1:(2*N+1),1:(2*N+1));
rud=sj(1:(2*N+1),(2*N+2):(4*N+2));
rdu=sj((2*N+2):(4*N+2),1:(2*N+1));
tdd=sj((2*N+2):(4*N+2),(2*N+2):(4*N+2));

Rxdu=Rdu+Tdd*rdu*inv(I-Rud*rdu)*Tuu;
Txuu=tuu*inv(I-Rud*rdu)*Tuu;
Rdu=Rxdu; Tuu=Txuu;

incident=zeros(2*N+1,1);
[amin,incI]=min(Y1-kz1(N+1));
incident(incI)=1;

% Reflection and transmission coefficient in x coordinate
Ttm=K*H3*Tuu*incident;
Rtm=K*H1*Rdu*incident;

% Diffraction efficiencies
DE1=zeros(1,2*N+1);
DE3=zeros(1,2*N+1);

for j=1:2*N+1
    DE1(j)=(abs(Rtm(j))^2)*real(kz1(j)/k1/cos(theta));
    DE3(j)=(abs(Ttm(j))^2)*real(n1^2*kz3(j)/(k1*(n3^2)*cos(theta)));
end
Transmssion = sum(DE3)
Reflection = sum(DE1);
